' Acquire.vbs
' A rudimentary image acquisition system for automated observatories
'
' NOTE: The telescope driver used is specified in an assignment statement
'		in the first few lines of the code.  This must be changed for your
'		hardware configuration; look for the variable TelescopeDriver.
'		The camera and focuser used are those selected in MaxIm DL.
'
'
' This script executes commands for slewing the telescope, taking images,
' etc., read from an input 'target list' file.  The available commands are:
'
'	DEST path_and_prefix
'		Set the destination directory (up to the last backslash) and
'		leading part of the filename for images to be saved in this run.
'		The default is the current directory, with empty prefix.
'
'	SEQ starting_number
'		Set the sequence number of the next image (incremented with each
'		image).  Executing a DEST command automatically sets the starting
'		number to one more than	the highest value that has been used with
'		that prefix that directory, so SEQ is only needed if you want to
'		overwrite existing files.
'
'	RA right_ascension
'		Set the right ascension for the next image.  It can be specified
'		as hours, hours and minutes, or hours, minutes and seconds, using
'		free-form delimiters.
'
'	DEC declination
'		Set the declination for the next image.  It can be specified
'		as degrees, degrees and minutes, or degrees, minutes and seconds,
'		using free-form delimiters.
'
'	EXP time
'		Take an exposure of the specified time (in seconds).  The result
'		image is saved in the DEST directory as prefix#####[_f].fts, where
'		prefix is the last part of the DEST string, ##### the five-digit
'		sequence number, and f is the first letter of the selected filter.
'		The underscore and filter letter do not appear if FILT is -1.
'
'	FILT filter_number
'		Set the filter number for the next exposure.  Range is -1 (filter
'		unchanged) and 0 through 12.  The default is -1.
'
'	FOCUS time
'		Perform an autofocus sequence using the specified exposure time.
'		You must have a focuser connected to use this command!
'
'	OBJ information
'		Set information to be inserted into OBJECT key of FITS header
'		on subsequent exposures.
'
' Commands are not case sensitive, and characters after those shown are
' ignored so that you can say "EXPOSE 10" instead of "EXP 10" if preferred.
' Blank lines and any characters following a pound sign (#) are ignored.


' declarations for all variables used globally
Dim fso
Dim Tel
Set Tel = Nothing
Dim Camera
Set Camera = Nothing
Dim Filter, Root, RA, Dec, HasRA, HasDec, ExpNo, Object
Filter = -1
Root = ""
Error = ""
HasRA = False
HasDec = False
Dim TelescopeDriver
TelescopeDriver = "ScopeSim.Telescope" 

' obtain name of target list
If wscript.Arguments.Count > 0 Then
	TargetList = wscript.Arguments(0)
Else
	TargetList = InputBox( "Enter name of target list file", "Acquire" )
End If

Set fso = CreateObject( "Scripting.FileSystemObject" )
If TargetList = "" Then
	wscript.echo "A target list file name is required"
	wscript.Quit
ElseIf Not fso.FileExists( TargetList ) Then
	wscript.echo "Target list file """ & TargetList & """ not found"
	wscript.Quit
End If

Set Util = CreateObject( "DriverHelper.Util" )

' read through the commands in the target list and do them one by one
Set f = fso.OpenTextFile( TargetList, 1 )
Do While Not f.AtEndOfStream
	Line = f.ReadLine
	N = N + 1
	Process( Replace( Util.TrimString( Line ), Chr(9), " " ))
	If Error <> "" Then wscript.echo Error & " at line " & N & ": " & Line
Loop
f.Close

wscript.echo "End of " & TargetList


' process a single command
Sub Process( Line )
	Error = ""
	Dim I, Command, Dur, F
	I = InStr( Line, "#" )						' anything after # is a comment
	If I > 0 Then Line = Left( Line, I - 1 )

	' the first word on the line is the command
	I = InStr( Line, " " )
	If I = 0 Then
		Command = UCase( Line )
		Line = ""
	Else
		Command = UCase( Left( Line, I - 1 ))
		Line = Util.TrimString( Right( Line, Len( Line ) - I ))
	End If

	If Left( Command, 2 ) = "RA" Then
		RA = Util.HMSToHours( Line )
		HasRA = RA <> 0 Or InStr( Line, "0" ) > 0
		If Not HasRA Then Error = "Missing right ascension"
	ElseIf Left( Command, 3 ) = "DEC" Then
		Dec = Util.DMSToDegrees( Line )
		HasDec = Dec <> 0 Or InStr( Line, "0" ) > 0
		If Not HasDec Then Error = "Missing declination"
	ElseIf Left( Command, 3 ) = "EXP" Then
		Dur = CSng( Line )
		If Dur > 0 And Dur <= 3600 Then
			ExpNo = ExpNo + 1
			TakeExposure( Dur )
		Else
			Error = "Invalid exposure time"
		End If
	ElseIf Left( Command, 4 ) = "FILT" Then
		F = CInt( Line )
		If F < -1 Or F > 12 Then
			Error = "Filter number out of range"
		ElseIf F = 0 And InStr( Line, "0" ) = 0 Then
			Error = "Missing filter number"
		Else
			Filter = F
		End If
	ElseIf Left( Command, 4 ) = "DEST" Then
		Root = Line
		ExpNo = GetSequenceNumber( Root )
	ElseIf Left( Command, 5 ) = "FOCUS" Then
		Dur = CSng( Line )
		If Dur > 0 And Dur < 60 Then
			AutoFocus( Dur )
		Else
			Error = "Invalid exposure time for autofocus"
		End If
	ElseIf Left( Command, 3 ) = "SEQ" Then
		F = CInt( Line )
		If F <= 0 Then
			Error = "Sequence number out of range"
		Else
			ExpNo = F - 1
		End If
	ElseIf Left( Command, 3 ) = "OBJ" Then
		Object = Line
		If Len( Object ) > 68 Then Object = Left( Object, 68 )
	ElseIf Command <> "" Then
		Error = "Unrecognized command"
	End If
End Sub

' take a single exposure of specified duration
Sub TakeExposure( Duration )
	PrepareToImage
	If Error <> "" Then Exit Sub

	' construct filename
	Dim Filename
	Filename = CStr( ExpNo )
	Filename = Root & Left( "00000", 5 - Len(Filename) ) & Filename
	On Error Resume Next		' in case Filter is out of range
	If Filter >= 0 Then
		Filename = Filename & "_" & Left( Camera.FilterNames( Filter ), 1 )
	End If
	On Error Goto 0
	Filename = Filename & ".fts"
'	wscript.echo "Exposing image " & Filename	'DEBUG

	' take requested exposure
	Camera.Expose Duration, 1, Filter
	Do
		wscript.Sleep 100
	Loop Until Camera.ImageReady

	' save the image
	If Object <> "" Then Camera.SetFITSKey "OBJECT", Object
	Camera.SaveImage Filename
End Sub

' perform an autofocus sequence and wait for it to complete
Sub AutoFocus( Duration )
	PrepareToImage
	If Error <> "" Then Exit Sub

	Dim App
	Set App = CreateObject( "MaxIm.Application" )
	App.AutoFocus Duration

	Do
		wscript.Sleep 100
	Loop While App.AutofocusStatus < 0

	If App.AutofocusStatus = 0 Then Error = "Autofocus failed"
End Sub

Sub PrepareToImage
	' connect the camera the first time we need it
	If Camera Is Nothing Then
		Set Camera = CreateObject( "MaxIm.CCDCamera" )
		Camera.DisableAutoShutdown = True
		Camera.LinkEnabled = True
		If Not Camera.LinkEnabled Then
			Error = "Failed to connect to camera"
			Exit Sub
		End If

		' filter names may not be available on the first exposure
		' unless MaxIm CCD is allowed to initialize fully
		wscript.Sleep 100
	End If

	' if RA and Dec have been set, slew there
	If HasRA And HasDec Then
		' connect telescope the first time
		If Tel Is Nothing Then
			Set Tel = CreateObject( TelescopeDriver )
			If Tel.Connected Then
				wscript.echo "NOTE: Telescope was already connected"
			Else
				Tel.Connected = True
			End If
			If Not Tel.CanSlew Then
				Error = "Selected telescope cannot slew"
				Exit Sub
			End If
			Tel.Unpark
			Tel.Tracking = True
		End If

'		wscript.echo "Slewing to " & RA & "," & Dec		'DEBUG
		Tel.SlewToCoordinates RA, Dec
		HasRA = False
		HasDec = False
	End If
End Sub

' determine the highest sequence number in use with a particular
' destination directory and filename prefix
Function GetSequenceNumber( P )
	Dim Dir, Pattern, I, J, RE, Matches, Files, fl, M

	' split the supplied Root into folder (before the last slash)
	' and filename prefix (after it)
	I = InStrRev( P, "/" )
	J = InStrRev( P, "\" )
	If I < J Then I = J
	If I > 0 Then
		Dir = Left( P, I - 1 )
		Pattern = Right( P, Len(P) - I )
		If Not fso.FolderExists( Dir ) Then
			Dim Folder
			On Error Resume Next
			Set Folder = fso.CreateFolder( Dir )
			If Folder Is Nothing Then
				Error = "Directory can't be created"
				GetSequenceNumber = ExpNo
			Else
				GetSequenceNumber = 0
			End If
			On Error Goto 0
			Exit Function
		End If
	Else
		Dir = "."		' no explicit folder, so use current directory
		Pattern = P
	End If

	' create a regular expression to match files having this prefix
	Pattern = "^" & Pattern & "([0-9][0-9][0-9][0-9][0-9]).*\.fts$"
	Set RE = New RegExp
	RE.Pattern = Pattern
	RE.IgnoreCase = True

	' find the largest sequence number of any file matching this pattern
	I = 0
	Set Files = fso.GetFolder( Dir ).Files
	For Each fl In Files
		Set Matches = RE.Execute( fl.Name )
		For Each M In Matches
			J = CInt( M.SubMatches(0) )
			If J > I Then I = J
		Next
	Next

	GetSequenceNumber = I
End Function
